@@ -19,23 +19,33 @@ T = TypeVar("T")
 
 
 class Pipeline(Generic[R]):
+    max_queue_size: int = 4
+    """max_queue_size
+    Change this value to alter the maximum number of item per pipeline element. Default
+    Default is 4
+    """
+
+    @staticmethod
+    def get_new_queue() -> "Queue[R]":
+        return Queue(maxsize=Pipeline.max_queue_size)
+
     def __init__(self, datas: Optional[Iterable[R]] = None):
-        self._source: "Queue[R]" = Queue()
+        self._source: "Queue[R]" = Pipeline.get_new_queue()
         self._parents: List["Process"] = []
-        if datas is not None:
-            self.source(datas)
         self.dispatcher = Dispatcher(self._source)
         self._register(self.dispatcher)
+        if datas is not None:
+            Timer(0.05, self.source, (datas,)).start()
 
     def source(self, datas: Iterable[R]) -> None:
         for data in datas:
             self._source.put(data)
 
-        Timer(0.01, self.auto_close).start()
+        self.auto_close()
 
     def auto_close(self):
         while not self._source.empty():
-            time.sleep(0.01)
+            time.sleep(0.15)
         self._source.put(GenericPipe.get_kill_word())
 
     def _register(self, process: Process):
@@ -47,7 +57,7 @@ class Pipeline(Generic[R]):
 
     def task(self, task: Callable[[R], S], count: int = 1) -> "Pipeline[S]":
         assert count >= 1
-        target_queue: "Queue[R]" = Queue()
+        target_queue: "Queue[R]" = Pipeline.get_new_queue()
         output_pipe: "Pipeline[S]" = Pipeline()
         self.dispatcher.dispatch_to([target_queue], count)
         for _ in range(count):
@@ -65,10 +75,9 @@ class Pipeline(Generic[R]):
         count: int = 1,
     ) -> "Pipeline[S]":
         assert count >= 1
-        target_queue: "Queue[R]" = Queue()
+        target_queue: "Queue[R]" = Pipeline.get_new_queue()
         output_pipe: "Pipeline[S]" = Pipeline()
         self.dispatcher.dispatch_to([target_queue], count)
-
         for _ in range(count):
             p = ClassInstancePipe(
                 target_queue,
@@ -83,7 +92,7 @@ class Pipeline(Generic[R]):
         return output_pipe
 
     def explode(self, explode_function: Callable[[R], Iterable[S]]) -> "Pipeline[S]":
-        target_queue: "Queue[R]" = Queue()
+        target_queue: "Queue[R]" = Pipeline.get_new_queue()
         output_pipe: "Pipeline[S]" = Pipeline()
         p = ExplodePipe(target_queue, explode_function, output_pipe._source)
 
@@ -92,7 +101,7 @@ class Pipeline(Generic[R]):
         return output_pipe
 
     def batch(self, count: int = 2) -> "Pipeline[Iterable[R]]":
-        target_queue: "Queue[R]" = Queue()
+        target_queue: "Queue[R]" = Pipeline.get_new_queue()
         output_pipe: "Pipeline[Iterable[R]]" = Pipeline()
         p = BatchPipe(target_queue, output_pipe._source, count)
         self.dispatcher.dispatch_to([target_queue])
@@ -101,7 +110,7 @@ class Pipeline(Generic[R]):
         return output_pipe
 
     def temporal_batch(self, time_interval: float) -> "Pipeline[Iterable[R]]":
-        target_queue: "Queue[R]" = Queue()
+        target_queue: "Queue[R]" = Pipeline.get_new_queue()
         output_pipe: "Pipeline[Iterable[R]]" = Pipeline()
         p = TimeBatchPipe(target_queue, output_pipe._source, time_interval)
         self.dispatcher.dispatch_to([target_queue])
@@ -110,7 +119,7 @@ class Pipeline(Generic[R]):
         return output_pipe
 
     def filter(self, filter_function: Callable[[R], bool]) -> "Pipeline[R]":
-        target_queue: "Queue[R]" = Queue()
+        target_queue: "Queue[R]" = Pipeline.get_new_queue()
         output_pipe: "Pipeline[R]" = Pipeline()
         p = FilterPipe(target_queue, filter_function, output_pipe._source)
         self.dispatcher.dispatch_to([target_queue])
@@ -129,33 +138,60 @@ class Pipeline(Generic[R]):
     def kill(self) -> None:
         self.dispatcher.terminate()
 
-    def prepare_output_buffer(self) -> "Queue[R]":
-        q: Queue[R] = Queue()
+    def _prepare_output_buffer(self) -> "Queue[R]":
+        q: Queue[R] = Pipeline.get_new_queue()
         self.dispatcher.dispatch_to([q])
         return q
 
-    def wait_for_completion(self, q: "Optional[Queue[R]]" = None) -> None:
-        if q is None:
-            q = self.prepare_output_buffer()
-        while True:
-            try:
-                packet = q.get()
-                if GenericPipe.is_death_packet(packet):
-                    return
-            except Exception as e:
-                print("Error", e)
-                return
+    # def wait_and_reduce(self,
+    #                     accumulator: T,
+    #                     reducer: Callable[[R, T], T]
+    #                     ) -> T:
+    #     self.
 
-    def wait_for_results(self, q: "Optional[Queue[R]]" = None) -> List[R]:
-        if q is None:
-            q = self.prepare_output_buffer()
-        output: List[R] = []
-        while True:
-            try:
-                packet = q.get()
+    @staticmethod
+    def wait_for_all_completions(
+        pipes: List["Pipeline[Any]"], timeout: Optional[int] = None
+    ) -> None:
+        output_queues: List[Queue[Any]] = [p._prepare_output_buffer() for p in pipes]
+
+        for o in output_queues:
+            while True:
+                packet = o.get(timeout=timeout)
                 if GenericPipe.is_death_packet(packet):
-                    return output
-                output.append(packet)
-            except Exception as e:
-                print("Error", e)
-                return output
+                    break
+
+    @staticmethod
+    def wait_for_all_results(pipes: List["Pipeline[Any]"]) -> List[List[Any]]:
+        output_queues: List[Optional[Queue[Any]]] = [
+            p._prepare_output_buffer() for p in pipes
+        ]
+
+        outputs: List[List[Any]] = [[] for _ in pipes]
+
+        while any(output_queues) > 0:
+            for i in range(len(output_queues)):
+                output_queue = output_queues[i]
+                if output_queue is None:
+                    continue
+                try:
+                    packet = output_queue.get(timeout=0.1)
+                    if GenericPipe.is_death_packet(packet):
+                        output_queues[i] = None
+                    else:
+                        outputs[i].append(packet)
+                except:
+                    pass
+
+        return outputs
+
+    def wait_for_completion(self, other_pipes: List["Pipeline[Any]"] = []) -> None:
+        return Pipeline.wait_for_all_completions([self, *other_pipes])
+
+    def wait_for_results(
+        self, other_pipes: List["Pipeline[Any]"] = []
+    ) -> List[List[R]]:
+        return Pipeline.wait_for_all_results([self, *other_pipes])
+
+    def wait_for_result(self) -> List[R]:
+        return Pipeline.wait_for_all_results([self])[0]
